#!/usr/bin/env python
# -*- cofing: utf-8 -*-
# vim:noet:sw=4:ts=4:ft=vim

'''
File: gtk-snippy.py
Descrition: Gtk Application for viewing and posting snippets on Snipplr.com
Author: Vladimir Kolev
'''

__revision__ = "1.1b"
__author__ = ["Vladimir Kolev <admin@vladimirkolev.com>"]


import sys, os
import random, string, time

try:
    from configobj import ConfigObj
    config = ConfigObj("data/pysnippy.conf")
except:
    print "Problem initializing the config file"

try:
    import pygtk
    pygtk.require('2.0')
    import gtk
    import gobject
    gobject.threads_init()
    import vte
except:
    print "You need pygtk installed to run the application\n"
    sys.exit(1)
try:
    from SnipplrPy import *
    sniplr = SnipplrPy()
    sniplr.setup(config['api_key'])
except:
    print "You need to install SnipplrPy! Please visit: http://geditsnipplr.googlecode.com/files/snipplrpy-0.4.tar.gz"
try:
    import pango
except:
    print "You need to install python-pango"
try:
    import sqlite3
except:
    print "python-sqlite3 is not installed, please install it"
try:
    import gtksourceview2 as gtksourceview
except:
    print "You need to install python-gtksourceview2"
    
import gettext
import locale

APP = 'pysnippy'
DIR = './locale'

locale.setlocale(locale.LC_ALL, '')
gettext.bindtextdomain(APP, DIR)
gettext.textdomain(APP)
_ = gettext.gettext

from lib import pysnipfuncs as psf
import base64

snippets = []

class GtkSnippy:

        def __init__(self):
                self.window = gtk.Window(gtk.WINDOW_TOPLEVEL)
                self.window.set_size_request(700, 450)
                self.window.set_title(_("PySnippy"))
                winicon = gtk.gdk.pixbuf_new_from_file('icons/pysnippy_22x22.png')
                self.window.set_icon(winicon)
                self.window.connect('delete_event', self.destroy)
                self.window.connect('destroy', self.destroy)

                self.hbox = gtk.HBox()

                self.toolbar = gtk.Toolbar()
                self.toolbar.set_icon_size(gtk.ICON_SIZE_LARGE_TOOLBAR)
                if int(config['mode']) == 0:
                    self.searchbox = gtk.combo_box_new_text()
                    self.searchbox.connect('changed', self.get_from_db)
                    results = psf.get_cats()
                    if results:
                        for cat in results:
                            self.searchbox.append_text(cat[0])
                    else:
                        print "Error"
                else:
                    self.searchbox = gtk.Entry()
                    self.searchbox.set_property('primary_icon_stock', gtk.STOCK_FIND)
                    self.searchbox.connect("activate", self.search_snipp)
                self.toolit = gtk.ToolItem()
                self.toolit.add(self.searchbox)
                self.addbutton = gtk.ToolButton(gtk.STOCK_ADD)
                self.addbutton.connect("clicked", self.add_snippet)
                self.delbutton = gtk.ToolButton(gtk.STOCK_DELETE)
                self.delbutton.connect("clicked", self.del_snippet)
                self.downbutton = gtk.ToolButton(gtk.STOCK_SAVE_AS)
                self.downbutton.connect("clicked", self.download_snippet)
                self.execbutton = gtk.ToolButton(gtk.STOCK_EXECUTE)
                self.execbutton.connect("clicked", self.exec_snipp)
                self.separator = gtk.SeparatorToolItem()
                self.separator2 = gtk.SeparatorToolItem()
                self.prefsbutton = gtk.ToolButton(gtk.STOCK_PREFERENCES)
                self.prefsbutton.connect("clicked", self.show_preferences)
                self.aboutbutton = gtk.ToolButton(gtk.STOCK_ABOUT)
                self.aboutbutton.connect("clicked", self.show_about)
                self.quitbutton = gtk.ToolButton(gtk.STOCK_QUIT)
                self.quitbutton.connect("clicked", self.destroy)


                self.toolbar.insert(self.toolit, 0)
                self.toolbar.insert(self.addbutton, 1)
                self.toolbar.insert(self.delbutton, 2)
                self.toolbar.insert(self.downbutton, 3)
                self.toolbar.insert(self.execbutton, 4)
                self.toolbar.insert(self.separator, 5)
                self.toolbar.insert(self.prefsbutton, 6)
                self.toolbar.insert(self.separator2, 7)
                self.toolbar.insert(self.aboutbutton, 8)
                self.toolbar.insert(self.quitbutton, 9)

                self.hbox.pack_start(self.toolbar, True, True, 0)


                self.hpanel = gtk.HPaned()
                
                store = self.create_model()
                
                lm = gtksourceview.LanguageManager()
                self.sourcebuffer = gtksourceview.Buffer()
                self.sourcebuffer.set_data('languages-manager', lm)
                mgr = gtksourceview.style_scheme_manager_get_default()
                self.sourcebuffer.set_highlight_syntax(True)
                style_scheme = mgr.get_scheme('oblivion')
                self.sourcebuffer.set_style_scheme(style_scheme)                
                
                swb = gtk.ScrolledWindow()
                swb.set_shadow_type(gtk.SHADOW_ETCHED_IN)
                swb.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
                self.browser = gtksourceview.View(self.sourcebuffer)
                font_desc = pango.FontDescription('monospace')
                self.browser.modify_font(font_desc)
                self.browser.set_show_line_numbers(True)
                self.browser.set_auto_indent(True)
                swb.add(self.browser)
                self.vbox2 = gtk.VBox()
                self.expander = gtk.Expander()
                self.expander.set_label(_("Comment: "))
                self.expander.set_expanded(True)
                self.comment_label = gtk.Label()
                self.comment_label.set_line_wrap(True)
                self.comment_label.set_alignment(0.0, 1.0)
                self.expander.add(self.comment_label)
                sw = gtk.ScrolledWindow()
                sw.set_shadow_type(gtk.SHADOW_ETCHED_IN)
                sw.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
                self.tree = gtk.TreeView(store)
                self.tree.set_vadjustment(None)
                self.tree.set_size_request(200, -1)
                self.tree.connect("row-activated", self.show_snipp)
                self.create_columns(self.tree)
                sw.add(self.tree)
                self.hpanel.add1(sw)
                self.vbox2.pack_start(swb)
                self.vbox2.pack_start(self.expander, False, False, 0)
                self.hpanel.add2(self.vbox2)


                self.vbox = gtk.VBox()
                self.vbox.pack_start(self.hbox, False, False, 0)
                self.vbox.pack_start(self.hpanel)
                self.vbox.show()
                self.window.add(self.vbox)
                self.window.show_all()

        def create_model(self):
                self.store = gtk.ListStore(str, str)

                for snip in snippets:
                        self.store.append([snip[0], snip[1]])

                return self.store

        def create_columns(self, treeView):
                """docstring for create_columns"""
                rendererText = gtk.CellRendererText()
                column = gtk.TreeViewColumn(_("Name"), rendererText, text=0)
                column.set_max_width(150)
                column.set_sort_column_id(0)
                treeView.append_column(column)

                rendererText = gtk.CellRendererText()
                column = gtk.TreeViewColumn(_("Language"), rendererText, text=1)
                column.set_sort_column_id(1)
                
                treeView.append_column(column)


        def destroy(self, widget, data=None):
                gtk.main_quit()

        def add_snippet(self, button):
                self.addsnip = gtk.Dialog()
                self.addsnip.set_icon(gtk.gdk.pixbuf_new_from_file('icons/pysnippy_22x22.png'))
                self.addsnip.set_title(_("Add snippet"))
                self.addsnip.set_size_request(500, 350)
                self.snipbuff = gtk.TextBuffer()
                catlabel = gtk.Label(_("Select category: "))
                catlabel.set_alignment(0, 1)
                self.categcombo = gtk.combo_box_new_text()
                results = psf.get_cats()
                for cat in results:
                    self.categcombo.append_text(cat[0])
                titlabel = gtk.Label(_("Enter title: "))
                titlabel.set_alignment(0, 1)
                self.titentry = gtk.Entry()
                sniplabel = gtk.Label(_("Enter your snippet: "))
                sniplabel.set_alignment(0, 1)
                sw = gtk.ScrolledWindow()
                sw.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
                self.snip = gtk.TextView(self.snipbuff)
                sw.add(self.snip)
                langlabel = gtk.Label(_("Select snippet language: "))
                langlabel.set_alignment(0, 1)
                self.langcombo = gtk.combo_box_new_text()
                
                desclabel = gtk.Label(_("Enter description: "))
                desclabel.set_alignment(0, 1)
                self.descentry = gtk.Entry()
                langs = psf.get_langs()
                for lang in langs:
                    self.langcombo.append_text(lang[0])
                self.addsnip.vbox.pack_start(catlabel, False, False, 0)
                self.addsnip.vbox.pack_start(self.categcombo, False, False, 0)
                self.addsnip.vbox.pack_start(titlabel, False, False, 0)
                self.addsnip.vbox.pack_start(self.titentry, False, False, 0)
                self.addsnip.vbox.pack_start(sniplabel, False, False, 0)
                self.addsnip.vbox.pack_start(sw, True, True, 0)
                self.addsnip.vbox.pack_start(langlabel, False, False, 0)
                self.addsnip.vbox.pack_start(self.langcombo, False, False, 0)
                self.addsnip.vbox.pack_start(desclabel, False, False, 0)
                self.addsnip.vbox.pack_start(self.descentry, False, False, 0)
                
                closebut = gtk.Button(gtk.STOCK_CLOSE, _("Close"))
                closebut.connect('clicked', self.close)
                savebut = gtk.Button(gtk.STOCK_SAVE, _("Save"))
                savebut.connect('clicked', self.store_to_db)
                
                self.addsnip.action_area.pack_start(closebut, False, False, 0)
                self.addsnip.action_area.pack_start(savebut, False, False, 0)
                
                self.addsnip.show_all() 
                
                
        def store_to_db(self, button):
            if config['mode'] == 0:
                catid = int(self.categcombo.get_active()) + 1
                title = self.titentry.get_text()
                snippet = self.snipbuff.get_text(self.snipbuff.get_start_iter(), self.snipbuff.get_end_iter())
                language = int(self.langcombo.get_active()) + 1
                description = self.descentry.get_text()
                #print "Inserting: ", catid, title, snippet, language, description
                if psf.storedb(catid, title, snippet, language, description) == 1:
                    pass
                else:
                    self.show_error(gtk.MESSAGE_ERROR, _("Error storing to the databse"))
                self.addsnip.destroy()
            else:
                title = self.titentry.get_text()
                snippet = base64.encodestring(self.snipbuff.get_text(self.snipbuff.get_start_iter(), self.snipbuff.get_end_iter()))
                language = self.langcombo.get_active_text()
                try:
                    sniplr.post(title, base64.decodestring(snippet), tags="PySnippy", language="%s" % language)
                except:
                    show_error(gtk.MESSAGE_ERROR, _("Problem storing your snippet in snipplr.com"))
                self.addsnip.destroy()
            
        def close(self, button):
            self.addsnip.destroy()

        def del_snippet(self, button):
                """docstring for del_snippet"""
                if config['mode'] == "online":
                    self.show_error(gtk.MESSAGE_INFO,
                                    _("Deleting is only possible in offline mode!"))
                else:
                    selection = self.tree.get_selection()
                    model, itern = selection.get_selected()
                    if itern == None:
                        self.show_error(gtk.MESSAGE_WARNING,
                                        _("You have to select something\nin order to delete it!"))
                    else:
                        snipid = model.get_value(itern, 0)
                        askdiag = gtk.MessageDialog(None, gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT, 
                                                    gtk.MESSAGE_QUESTION, gtk.BUTTONS_YES_NO, 
                                                    _("Are you sure you want to delete the snippet?"))
                        askdiag.format_secondary_markup(_("Snippet: <b><i>%s</i></b>") % snipid)
                        answer = askdiag.run()
                        if answer == gtk.RESPONSE_YES:
                            if psf.delete_snip(snipid) == 1:
                                print _("Snippet %s deleted") % snipid
                            else:
                                self.show_error(gtk.MESSAGE_ERROR, _("Problem deleting snippet"))
                                
                        else:
                            askdiag.destroy()
                        askdiag.destroy()

        def download_snippet(self, button):
                """docstring for download_snippet"""
                if int(config['mode']) == 1:
                    selection = self.tree.get_selection()
                    model, itern = selection.get_selected()
                    if itern != None:
                        snipid = model.get_value(itern, 0)
                        catdiag = gtk.MessageDialog(None, gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT,
                                                    gtk.MESSAGE_QUESTION, gtk.BUTTONS_OK)
                        catdiag.set_markup(_('Please select a <b>category</b>'))
                        combo = gtk.combo_box_new_text()
                        results = psf.get_cats()
                        for cat in results:
                            combo.append_text(cat[0])
                        combo.set_active(0)
                        hbox = gtk.HBox()
                        hbox.pack_start(gtk.Label(_("Category: ")), False, 5, 5)
                        hbox.pack_start(combo)
                        catdiag.format_secondary_markup(_("Default is <i>scripts</i>"))
                        catdiag.vbox.pack_start(hbox, True, True, 0)
                        catdiag.show_all()
                        catdiag.run()
                        catid = int(combo.get_active()) + 1
                        catdiag.destroy()
                        snippetinf = sniplr.get(snipid)
                        sniptitle = snippetinf['title']
                        snipsource = snippetinf['source']
                        sniplang = psf.get_langid(snippetinf['language'].lower())
                        print snippetinf['language'].lower()
                        print sniplang
                        snipcomment = snippetinf['comment']
                        if snipcomment == "":
                            snipcomment = "Downloaded from sniplr.com"
                        try:
                            psf.storedb(catid, sniptitle, snipsource, sniplang, snipcomment)
                        except:
                            self.show_error(gtk.MESSAGE_ERROR, _("Error storing data to the database"))
                                                
                    else:
                        self.show_error(gtk.MESSAGE_INFO, _("Nothing to download!"))
                else:
                   self.show_error(gtk.MESSAGE_INFO,
                                   _("You can't download snippet when\nyou are using PySnippy in offline mode!"))

        def exec_snipp(self, button):                
                """docstring for exec_snipp"""
                selection = self.tree.get_selection()
                model, itern = selection.get_selected()
                if itern == None:
                    self.show_error(gtk.MESSAGE_WARNING, _("Sorry, there is nothing to execute!"))
                else:
                    snipid = model.get_value(itern, 0)
                    if int(config['mode']) == 0:
                        results = psf.get_snip_fromdb(snipid)
                        for result in results:
                            if self.get_lang(result[2]) == "python":
                                try:
                                    exec base64.decodestring(result[0])
                                except:
                                    self.show_error(gtk.MESSAGE_ERROR, _("Can't execute script"))                                                        
                            else:
                                self.show_error(gtk.MESSAGE_WARNING, _("Only python scripts can be executed!"))
                    else:
                        if sniplr.get(snipid)['language'] == "python":
                            print "Executing python script"
                        else:
                            self.show_error(gtk.MESSAGE_INFO, _("Only python scripts can be executed!"))
                    
        def gen_snip_name(self):
            sampling = "".join(random.sample(string.letters + string.digits, 6))
            return sampling
            

        def show_preferences(self, button):
                """docstring for show_preferences"""
                prefs = gtk.Dialog(_("Preferences"), None, gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT,
                                   (gtk.STOCK_CANCEL, gtk.RESPONSE_REJECT, gtk.STOCK_APPLY, gtk.RESPONSE_ACCEPT))
                hbox1 = gtk.HBox()
                apilab = gtk.Label(_("Snipplr.com API_KEY: "))
                apient = gtk.Entry()
                apient.set_text(config['api_key'])
                hbox1.pack_start(apilab, True, False)
                hbox1.pack_start(apient, False, False)
                hbox2 = gtk.HBox()
                modelab = gtk.Label(_("Working mode: "))
                modechk = gtk.CheckButton(_("Online"))
                modechk.set_active(int(config['mode']))
                hbox2.pack_start(modelab, False, False)
                hbox2.pack_start(modechk, True, False)
                prefs.vbox.pack_start(hbox1)
                prefs.vbox.pack_start(hbox2)
                prefs.show_all()
                response = prefs.run()
                if response == gtk.RESPONSE_ACCEPT:
                    config['api_key'] = apient.get_text()
                    config['mode'] = int(modechk.get_active())
                    config.write()
                else:
                    prefs.destroy()
                prefs.destroy()
                

        def show_snipp(self, tree, ident, column):
                """docstring for show_snipp"""
                selection = tree.get_selection()
                model, itern = selection.get_selected()
                snipid = model.get_value(itern, 0)
                if int(config['mode']) == 0:
                    results = psf.get_snip_fromdb(snipid)
                    for result in results:
                        self.sourcebuffer.set_text(base64.decodestring(result[0]))
                        self.comment_label.set_text(result[1])
                        sniplang = self.get_lang(result[2])
                else:              
                    snipsource = sniplr.get(snipid, True)['source']
                    snipcomment = sniplr.get(snipid)['comment']
                    #snipsource = snipsource.replace("&quot;", "'").replace("&amp;", "&").replace("&lt;", "<").replace("&gt;", ">")
                    sniplang = sniplr.get(snipid)['language']
                    self.comment_label.set_text(snipcomment.replace("&quot;", "'").replace("&amp;", "&").replace("&lt;", "<").replace("&gt;", ">"))
                    self.sourcebuffer.set_text(snipsource)
                                    
                manager = self.sourcebuffer.get_data('languages-manager')
                language = manager.get_language(sniplang.lower())
                self.sourcebuffer.set_language(language)
                self.sourcebuffer.set_highlight_syntax(True)
                
                

        def show_about(self, button):
                """docstring for show_about"""
                about = gtk.AboutDialog()
                about.set_icon(gtk.gdk.pixbuf_new_from_file('icons/pysnippy_22x22.png'))
                about.set_program_name(_("PySnippy"))
                about.set_version(__revision__)
                about.set_logo(gtk.gdk.pixbuf_new_from_file('icons/pysnippy_128x128.png'))
                about.set_comments(_("PySnippy is a gtk application for managing your snippet localy and on snipplr.com"))
                about.set_website('https://bitbucket.org/vkolev/pysnippy')
                about.set_authors(__author__)
                about.set_copyright(_("Licensed under MIT License"))
                try:
                        f = open('LICENSE', 'r')
                        about.set_license(f.read())
                        f.close()
                except:
                        self.show_error(gtk.MESSAGE_ERROR, _("Problem reading the LICENSE file"))
                about.run()
                about.destroy()

        def search_snipp(self, widget):
                if self.searchbox.get_text() == "":
                        pass
                else:
                        sniplrs = sniplr.list(self.searchbox.get_text())
                        self.store.clear()
                        for snip in sniplrs:
                                self.store.append((snip['id'], snip['title']))
                                
        def get_from_db(self, widget):
            catid = int(self.searchbox.get_active()) + 1
            conn = sqlite3.connect('data/snippets.db')
            c = conn.cursor()
            results = c.execute("""select name, language from snippets where cat_id = %s""" % (catid))
            self.store.clear()
            for result in results:
                self.store.append((result[0], self.get_lang(result[1])))
                
        def get_lang(self, langid):
            conn = sqlite3.connect('data/snippets.db')
            c = conn.cursor()
            result = c.execute("""select language from languages where id = %s""" % (langid))
            for res in result:
                return res[0]
                        
        def get_language_for_mime_type(self, mime):
            lang_manager = gtksourceview.language_manager_get_default()
            lang_ids = lang_manager.get_language_ids()
            for i in lang_ids:
                lang = lang_manager.get_language(i)
                for m in lang.get_mime_types():
                    if m == mime:
                        return lang
            return None
        
        def show_error(self, type=gtk.MESSAGE_WARNING, message=""):
            dialog = gtk.MessageDialog(None, gtk.DIALOG_DESTROY_WITH_PARENT,
                                       gtk.MESSAGE_WARNING, gtk.BUTTONS_CLOSE,
                                       message)
            dialog.run()
            dialog.destroy()

        def main(self):
                gtk.main()


if __name__ == "__main__":
        snippy = GtkSnippy()
        snippy.main()
